using System;
using System.Text;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text.RegularExpressions;
using System.Xml;


namespace OktellPlugin
{
	#region CMapperXmlTransformer
	/// <summary>
	/// ,     
	///      XML-  
	/// </summary>
	public class CMapperXmlTransformer
	{
		/// <summary>
		///     .  
		/// </summary>
		private CMapperXmlTransformer ( )
		{
		}

		#region Tags

		//  
		private static string m_tagcoll1 = "property_collection";
		private static string m_tagset1 = "property_set";		
		private static string m_tagpropsimple1 = "property_simple";
		private static string m_tagpropcdata1 = "property_cdata";
		private static string m_attkey1 = "key";
		private static string m_attvalue1 = "value";
		private static string m_attname1 = "name";
		private static string m_attcount1 = "count";

		//  
		private static string m_tagcoll2 = "prc";
		private static string m_tagset2 = "prs";
		private static string m_tagpropsimple2 = "s";
		private static string m_tagpropcdata2 = "c";
		private static string m_attkey2 = "k";
		private static string m_attvalue2 = "v";
		private static string m_attname2 = "n";
		private static string m_attcount2 = "cn";

		//    
		private static string m_tagstart = "oktellxmlmapper";		
		private static string m_tagdata = "data";		
		private static string m_tagcoll = "property_collection";
		private static string m_tagset = "property_set";
		private static string m_tagpropsimple = "property_simple";
		private static string m_tagpropcdata = "property_cdata";
		private static string m_attkey = "key";
		private static string m_attvalue = "value";
		private static string m_attname = "name";
		private static string m_attcount = "count";

	
		/// <summary>
		///    TagType (   )
		/// </summary>
		public enum EMapperXmlTagType
		{
			Full	= 0,
			Short	= 1
		}

		/// <summary>
		///      
		/// </summary>
		private static EMapperXmlTagType m_CurrentTagType = EMapperXmlTagType.Full;

		/// <summary>
		///     
		/// </summary>
		/// <returns></returns>
		public static void SetCurrentTagType ( EMapperXmlTagType type )
		{
			m_CurrentTagType = type;

			switch ( type )
			{
				case EMapperXmlTagType.Full:
					m_tagcoll = m_tagcoll1;	
					m_tagset = m_tagset1;									
					m_tagpropsimple = m_tagpropsimple1;
					m_tagpropcdata = m_tagpropcdata1;
					m_attkey = m_attkey1;
					m_attvalue = m_attvalue1;
					m_attname = m_attname1;
					m_attcount = m_attcount1;
					break;

				case EMapperXmlTagType.Short:
					m_tagcoll = m_tagcoll2;
					m_tagset = m_tagset2;
					m_tagpropsimple = m_tagpropsimple2;
					m_tagpropcdata = m_tagpropcdata2;
					m_attkey = m_attkey2;
					m_attvalue = m_attvalue2;
					m_attname = m_attname2;
					m_attcount = m_attcount2;
					break;
			}
		}

		/// <summary>
		///     
		/// </summary>
		private static EMapperXmlTagType GetCurrentTagType ( )
		{
			return m_CurrentTagType;
		}
		#endregion

		#region Encoding
		/// <summary>
		///   
		/// </summary>
		public enum EMapperXmlEncoding
		{
			Default					= 0,
			ASCII					= 1,
			BigEndianUnicode		= 2,
			UTF7					= 3,
			UTF8					= 4,
			UTF16					= 5
		}

		/// <summary>
		///   
		/// </summary>
		private static Encoding m_CurrentEncoding = Encoding.Unicode;

		/// <summary>
		///      
		/// </summary>
		/// <returns></returns>
		public static void SetCurrentEncoding ( EMapperXmlEncoding type )
		{
			switch ( type )
			{
				case EMapperXmlEncoding.Default:			m_CurrentEncoding = Encoding.Default; break;
				case EMapperXmlEncoding.ASCII:				m_CurrentEncoding =  Encoding.ASCII; break;
				case EMapperXmlEncoding.BigEndianUnicode:	m_CurrentEncoding =  Encoding.BigEndianUnicode; break;
				case EMapperXmlEncoding.UTF7:				m_CurrentEncoding =  Encoding.UTF7; break;
				case EMapperXmlEncoding.UTF8:				m_CurrentEncoding =  Encoding.UTF8; break;
				case EMapperXmlEncoding.UTF16:				m_CurrentEncoding =  Encoding.Unicode; break;
				default:									m_CurrentEncoding =  Encoding.Unicode; break;
			}
		}

		/// <summary>
		///     
		/// </summary>
		private static Encoding GetCurrentEncoding ( )
		{
			return m_CurrentEncoding;			
		}
		#endregion

		#region Write

		/// <summary>
		///    XML        
		/// </summary>
		/// <returns></returns>
		private static double GetVersion ( )
		{
			return 080710;
		}

		#region Inner Xml
		/// <summary>
		///  MemoryStream,          DATA
		/// </summary>
		/// <returns>  XML</returns>
		private static XmlTextWriter _CreateXmlHeader ( MemoryStream ms )
		{	
			XmlTextWriter writer = new XmlTextWriter ( ms, GetCurrentEncoding() );
			writer.Formatting = Formatting.Indented;			
			
			//document
			writer.WriteStartDocument ( );
			writer.WriteStartElement ( m_tagstart );
			writer.WriteAttributeString ( "version", GetVersion().ToString ( ) );
			
			//data
			writer.WriteStartElement ( m_tagdata );

			return writer;
		}

		/// <summary>
		///  MemoryStream,          DATA
		/// </summary>
		/// <returns>  XML</returns>
		private static XmlTextWriter _CreateXmlHeader ( string path )
		{	
			XmlTextWriter writer = new XmlTextWriter ( path, GetCurrentEncoding() );
			writer.Formatting = Formatting.Indented;			
			
			//document
			writer.WriteStartDocument ( );
			writer.WriteStartElement ( m_tagstart );
			writer.WriteAttributeString ( "version", GetVersion().ToString ( ) );
			
			//data
			writer.WriteStartElement ( m_tagdata );

			return writer;
		}
		
		/// <summary>
		///   XML   /DATA   
		/// </summary>
		private static void _FinishXml ( XmlTextWriter writer )
		{
			//Data
			writer.WriteEndElement ( );

			//oktellcommapper
			writer.WriteEndElement ( );
			//document
			writer.WriteEndDocument ( );
			
			writer.Close ( );
		}
		
		/// <summary>
		///   xml   .
		///       
		/// </summary>
		private static void _InsertXmlSet ( XmlTextWriter writer, CMapperXmlItemSet items )
		{			
			if ( items != null )
			{
				//     XML  
				#region   
				foreach ( CMapperXmlItem e in items )
				{				
					//  ( key="newstate")
					string key = e.Key;
					//  ( value="5")
					string val = e.Value;
					//  ( name="fullbusy")
					string text = e.Name;				
				
					switch ( e.Type )
					{
							#region Simple							
						case CMapperXmlItem.EItemType.Simple:
						{
							//<property_simple key="" value="" name=""/>
							writer.WriteStartElement ( m_tagpropsimple );
				
							if ( key != null )
								writer.WriteAttributeString ( "key", key );
							if ( val != null )
								writer.WriteAttributeString ( "value", val );
							if ( text != null )
								writer.WriteAttributeString ( "name", text );
				
							writer.WriteEndElement ( );

							break;
						}
							#endregion

							#region CData							
						case CMapperXmlItem.EItemType.CData:
						{
							//<property_cdata key="" value="">[![CDATA[""]]</property_cdata>
							writer.WriteStartElement ( m_tagpropcdata );
				
							if ( key != null )
								writer.WriteAttributeString ( "key", key );
							if ( val != null )
								writer.WriteAttributeString ( "value", val );
							
							if ( text != null )
							{
								if ( text.IndexOf ( "]]" ) >= 0 )
								{
									//11.08.09									
									writer.WriteAttributeString ( "content-type", "base64" );
									writer.WriteCData ( Convert.ToBase64String ( m_CurrentEncoding.GetBytes ( text ) ) );
								}
								else
								{
									try
									{
										writer.WriteCData ( text );
									}
									catch 
									{	
										//11.08.09
										string b64pref = "(#b64#)_";
										writer.WriteCData ( b64pref + Convert.ToBase64String ( m_CurrentEncoding.GetBytes ( text ) ) );
									}
								}
							}
				
							writer.WriteEndElement ( );

							break;
						}
							#endregion
					}
				}
				#endregion

				#region 
				if ( items.SubSets.Count > 0 )
				{
					// 
					writer.WriteStartElement ( m_tagcoll );
					string name = items.SubSets.Name;
					if (( name != null ) && ( name != String.Empty ))
						writer.WriteAttributeString ( "name", name );
					writer.WriteAttributeString ( "count", items.SubSets.Count.ToString() );

					// 
					_InsertXmlColl ( writer, items.SubSets );

					// 
					writer.WriteEndElement ( );
				}
				#endregion
			}			
		}

		/// <summary>
		///   xml   .
		///      
		/// </summary>
		private static void _InsertXmlColl ( XmlTextWriter writer, CMapperXmlItemSetCollection items )
		{			
			if ( items != null )
			{
				//     XML  
				foreach ( CMapperXmlItemSet e in items )
				{
					// 
					writer.WriteStartElement ( m_tagset );
					if (( e.Name != null ) && ( e.Name != String.Empty ))
						writer.WriteAttributeString ( "name", e.Name );
					if (( e.IdName != null ) && ( e.IdName != String.Empty ))
						writer.WriteAttributeString ( "id", e.IdName );

					// 
					_InsertXmlSet ( writer, e );
					
					// 
					writer.WriteEndElement ( );
				}
			}			
		}

		#endregion 

		#region GetXmlString (+ overloads)
		
		/// <summary>
		///  XML ,        - 
		/// </summary>
		/// <param name="eventtype"> </param>
		/// <param name="items">  CMapperXmlItem</param>		
		/// <returns>  UTF-16,  XML- </returns>
		public static string GetXmlString ( string description, CMapperXmlItemSet items )
		{
			MemoryStream ms = new MemoryStream ( );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( ms );
			
			//   <data>
			writer.WriteAttributeString ( "name", description );
						
			// 
			_InsertXmlSet ( writer, items );
			
			//
			_FinishXml ( writer );

			string xmlstring = GetCurrentEncoding().GetString ( ms.ToArray () );
			
			//   -  ..

			return xmlstring;
		}

		/// <summary>
		///   XML        
		/// </summary>
		/// <param name="description">,     name</param>
		/// <param name="items"> </param>
		/// <returns>  UTF-16,  XML- </returns>
		public static string GetXmlString ( string description, CMapperXmlItemSetCollection items )
		{
			MemoryStream ms = new MemoryStream ( );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( ms );
			
			int itemscnt = 0;
			if ( items != null )
				itemscnt = items.Count;
			
			//   <data>
			writer.WriteAttributeString ( "name", description );
			writer.WriteAttributeString ( "count", itemscnt.ToString() );
			
			// 
			_InsertXmlColl ( writer, items );
						
			//
			_FinishXml ( writer );

			
			string xmlstring = GetCurrentEncoding().GetString ( ms.ToArray () );
			
			//   -  ..

			return xmlstring;
		}		

		/// <summary>
		///   XML (  )        
		/// </summary>
		/// <param name="description">,     name</param>
		/// <param name="items"> </param>
		/// <param name="items">  </param>
		public static void CreateXmlFile ( string description, CMapperXmlItemSetCollection items, string path )
		{
			// -
			if ( File.Exists ( path ) )
				File.Delete ( path );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( path );
						
			//   <data>
			writer.WriteAttributeString ( "name", description );
			writer.WriteAttributeString ( "count", items.Count.ToString() );
			
			// 
			_InsertXmlColl ( writer, items );
						
			//
			_FinishXml ( writer );			
		}
		#endregion

		#endregion

		#region Parse

		#region Inner
		
		/// <summary>
		///  ,   ,
		///      
		/// </summary>
		/// <param name="reader">XmlTextReader        </param>
		/// <param name="an1"> 1 </param>
		/// <param name="an2"> 2 </param>
		/// <returns>   NULL   </returns>
		private static string GetAttribute ( XmlTextReader reader, string an1, string an2 )
		{
			string ret = reader.GetAttribute ( an1 );
			if ( ret == null )
				ret = reader.GetAttribute ( an2 );
			
			return ret;
		}

		/// <summary>
		///      
		/// XML-  XmlTextReader   
		/// </summary>
		/// <param name="reader">XmlTextReader   </param>
		/// <param name="basecoll"></param>
		private static void _ParseDataChapter ( XmlTextReader reader, CMapperXmlItemSetCollection basecoll )
		{
			CMapperXmlItemSetCollection currentColl = basecoll;
			CMapperXmlItemSet currentSet = null;
			
			// ,     ( DATA = SET )
			CMapperXmlItemSet errorset = new CMapperXmlItemSet ( );
			errorset.ParentCollection = basecoll;			
			errorset.Name = GetAttribute ( reader, m_attname1, m_attname2 );
			basecoll.Add ( errorset );
			currentSet = errorset;

			while ( ! reader.EOF )
			{
				reader.Read ( );
				while (( ! reader.EOF ) && 
					( reader.NodeType != XmlNodeType.Element ) &&
					( reader.NodeType != XmlNodeType.EndElement ))
					reader.Read ( );

				if ( reader.EOF )
					break;
				
				switch ( reader.NodeType )
				{
					case XmlNodeType.Element:
						#region Element
						if (( reader.Name == m_tagcoll1 ) || 
							( reader.Name == m_tagcoll2 ))
						{
							string name = GetAttribute ( reader, m_attname1, m_attname2 );
							string count = GetAttribute ( reader, m_attcount1, m_attcount2 );

							//      , 
							//      data   
							CMapperXmlItemSetCollection setcoll = null;
							if (( currentColl == basecoll ) && ( currentSet == null ))
								setcoll = basecoll;
							else							
								setcoll = currentSet.SubSets;							
					
							setcoll.Name = name;
							currentColl = setcoll;
						}
						else if (( reader.Name == m_tagset1 ) || 
							( reader.Name == m_tagset2 ))
						{
							// , DATA=COLL,    ERRORSET
							if (( currentSet == errorset ) && ( currentColl == basecoll ))
								basecoll.Remove ( errorset );

							string name = GetAttribute ( reader, m_attname1, m_attname2 );
							string idname = reader.GetAttribute ( "id" );
							
							CMapperXmlItemSet itemset = new CMapperXmlItemSet ( );
							itemset.Name = name;
							itemset.IdName = idname;
							itemset.ParentCollection = currentColl;

							if ( ! reader.IsEmptyElement )							
								currentSet = itemset;
							
							currentColl.Add ( itemset );
						}
						else if (( reader.Name == m_tagpropsimple1 ) || 
							( reader.Name == m_tagpropsimple2 ))
						{
							string key = GetAttribute ( reader, m_attkey1, m_attkey2 );
							string val = GetAttribute ( reader, m_attvalue1, m_attvalue2 );
							string name = GetAttribute ( reader, m_attname1, m_attname2 );

							CMapperXmlItem item = new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key, val, name );

							currentSet.Add ( item );
						}
						else if (( reader.Name == m_tagpropcdata1 ) || 
							( reader.Name == m_tagpropcdata2 ))
						{
							string key = GetAttribute ( reader, m_attkey1, m_attkey2 );
							string val = GetAttribute ( reader, m_attvalue1, m_attvalue2 );
							string ct = GetAttribute ( reader, "content-type", "ct" );

							string text = null;
							if ( ! reader.IsEmptyElement )
							{
								XmlNodeType type = reader.MoveToContent ( );
								if ( type == XmlNodeType.CDATA )
									text = reader.Value;
								else if ( type == XmlNodeType.Element )
								{
									while (( reader.NodeType != XmlNodeType.CDATA ) &&
										( reader.NodeType != XmlNodeType.EndElement ))
										reader.Read();
							
									if ( reader.NodeType == XmlNodeType.CDATA )
										text = reader.Value;
								}
							}

							//11.08.09
							string b64pref = "(#b64#)_";
							if ( ct != null )
							{
								if ( ct.ToLower() == "base64" )								
									text = m_CurrentEncoding.GetString ( Convert.FromBase64String ( text ) );
							}
							else if ( text.StartsWith ( b64pref ) )
							{
								text = text.Remove ( 0, b64pref.Length );
								text = m_CurrentEncoding.GetString ( Convert.FromBase64String ( text ) );
							}
							
							CMapperXmlItem item = new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key, val, text );

							currentSet.Add ( item );
						}
						break;
						#endregion

					case XmlNodeType.EndElement:
						#region EndElement
						if (( reader.Name == m_tagcoll1 ) ||
							( reader.Name == m_tagcoll2 ))
						{
							currentColl = currentSet.ParentCollection;
						}
						else if (( reader.Name == m_tagset1 ) ||
							( reader.Name == m_tagset2 ))
						{
							currentSet = currentColl.ParentSet;
						}
						break;
						#endregion
				}
			}
		}
		
		#endregion

		#region ParseXmlString

		/// <summary>
		///         
		/// </summary>
		/// <param name="reader">XmlTextReader   </param>
		/// <returns>  </returns>
		private static CMapperXmlItemSetCollection _ParseXmlReader ( XmlTextReader reader )
		{
			if ( reader == null )
				return null;

			// 
			CMapperXmlItemSetCollection coll = new CMapperXmlItemSetCollection ( );
			double version = GetVersion ( );

			//        (      )
			while ( ! reader.EOF )
			{
				reader.Read ( );
				while (( ! reader.EOF ) && ( reader.NodeType != XmlNodeType.Element ))
					reader.Read ( );

				if ( reader.EOF )
					break;

				if ( reader.Name.ToLower() == m_tagstart )
					version = double.Parse ( reader.GetAttribute ( "version" ) );
				
				else if ( reader.Name.ToLower() == m_tagdata )					
				{	
					coll.Name = GetAttribute ( reader, m_attname1, m_attname2 );

					_ParseDataChapter ( reader, coll );						
				}						
			}

			return coll;
		}

		/// <summary>
		///      XML-
		/// </summary>
		/// <param name="xml">  XML   </param>
		/// <returns>  </returns>
		public static CMapperXmlItemSetCollection ParseXmlString ( string xml )
		{
			//  
			if ( xml == null )			
				return null;			
			
			XmlTextReader reader = null;

			try
			{
				//    XmlTextReader
				MemoryStream ms = new MemoryStream ( GetCurrentEncoding().GetBytes ( xml ) );
				reader = new XmlTextReader ( ms );
				reader.WhitespaceHandling = WhitespaceHandling.None;

				return _ParseXmlReader ( reader );
			}
			catch ( Exception ex )
			{
				//Log.Write ( ex.ToString() );
			}
			finally
			{
				if ( reader != null )
					reader.Close ( );
			}
			
			return null;
		}

		/// <summary>
		///      XML-
		/// </summary>
		/// <param name="path">  </param>
		/// <returns>  </returns>
		public static CMapperXmlItemSetCollection ParseXmlFile ( string path )
		{
			//  
			if ( path == null )			
				return null;
			
			if ( ! File.Exists ( path ) )
				return null;

			XmlTextReader reader = null;

			try
			{
				reader = new XmlTextReader ( path );
				reader.WhitespaceHandling = WhitespaceHandling.None;

				return _ParseXmlReader ( reader );
			}
			catch ( Exception ex )
			{
				//Log.Write ( ex.ToString() );
			}
			finally
			{
				if ( reader != null )
					reader.Close ( );
			}

			return null;
		}
		#endregion

		#endregion
	}
	#endregion

	#region CMapperXmlItem
	/// <summary>
	///  XML-,        .
	///  XML-   property_simple  property_cdata
	///  -,  /  .
	///              XML-,
	///      CDATA
	/// </summary>
	[Serializable]
	public class CMapperXmlItem
	{
		/// <summary>
		///    . 
		/// </summary>
		public enum EItemType
		{
			Simple,		//<property_simple key="" value="" name=""/>
			CData		//<property_cdata key="" value="">[![CDATA[""]]</property_cdata>
		}

		private EItemType m_Type = EItemType.Simple;
		/// <summary>
		///   . 
		///      XML-. 
		///    (simple)     CDATA (CData)
		/// </summary>
		public EItemType Type
		{
			get { return m_Type; }
		}

		/// <summary>
		/// - 
		/// </summary>
		public string Key = null;
		
		/// <summary>
		///  
		/// </summary>
		public string Value = null;
		
		/// <summary>
		///   
		/// </summary>
		public string Name = null;

		public CMapperXmlItem ( EItemType type, string key, string val, string name )
		{
			m_Type = type;

			Key = key;
			Value = val;
			Name = name;
		}
		
		public CMapperXmlItem ( EItemType type )
		{
			m_Type = type;
		}

		private static Regex regex;

		/// <summary>
		///   .
		///     ( CMapperXmlItem)
		///       
		///            .
		/// </summary>
		/// <param name="key"> </param>
		/// <param name="val">  (   "VALUE").        
		///      -    "NAME"</param>
		/// <returns>    </returns>
		public static CMapperXmlItem GetItem ( string key, object val )
		{
			if ( regex == null )
				regex = new Regex ( @"[^A-Za-z0-9]+" );

			if ( val != null )
			{
				if ( val is Guid )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), null );

				if ( val is Boolean )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((bool)val) ? "1" : "0", null );
				
				else if ( val is Enum )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), Convert.ToInt32 ( val ).ToString(), val.ToString() );
				
				else if ( val is DateTime )
				{
					DateTime dt = (DateTime)val;

					if ( dt.Year >= 1980 )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), dt.ToString ( "dd.MM.yyyy HH:mm:ss" ), null );
					else
					{						
						TimeSpan sp = new TimeSpan ( dt.Ticks );
						string h = ((int) Math.Floor ( sp.TotalHours )).ToString("00");
						string m = sp.Minutes.ToString("00");
						string s = sp.Seconds.ToString("00");
						
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), h + ":" + m + ":" + s, null );
					}
				}
				
				else if ( val is TimeSpan )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), null );
				
				else if ( val is string )
				{
					string sval = val as String;
					if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), sval, null );
				}
				
				else
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), null );
			}

			return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), String.Empty, null );
		}

		/// <summary>
		///   .
		///     ( CMapperXmlItem)
		///       
		///            .
		/// </summary>
		/// <param name="key"> </param>
		/// <param name="val"> </param>
		/// <param name="name"> </param>
		/// <returns>    </returns>
		public static CMapperXmlItem GetItem ( string key, object val, string name )
		{
			if ( regex == null )
				regex = new Regex ( @"[^A-Za-z0-9]+" );

			if ( val != null )
			{				
				if (( name != null ) && 
					(( regex.IsMatch ( name ) ) || ( name.Length > 50 )))
				{
					#region Val , Name - cdata
					
					if ( val is Guid )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), val.ToString(), name );
				
					else if ( val is Enum )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), Convert.ToInt32 ( val ).ToString(), name );
				
					else if ( val is DateTime )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), ((DateTime)val).ToString ( "dd.MM.yyyy HH:mm:ss" ), name );
				
					else if ( val is TimeSpan )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), name );
				
					else if ( val is string )
					{
						string sval = val as String;
						if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
						else
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), sval, name );
					}
				
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), val.ToString(), name );
					
					#endregion
				}
				else
				{
					#region Val , name 

					if ( val is Guid )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), name );
				
					else if ( val is Enum )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), Convert.ToInt32 ( val ).ToString(), name );
				
					else if ( val is DateTime )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((DateTime)val).ToString ( "dd.MM.yyyy HH:mm:ss" ), name );
				
					else if ( val is TimeSpan )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), name );
				
					else if ( val is string )
					{
						string sval = val as String;
						if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
						else
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), sval, name );
					}
				
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), name );

					#endregion
				}
			}

			else
			{
				#region Val  

				if ( name != null )
				{
					if (( regex.IsMatch ( name ) ) || ( name.Length > 50 ))				
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, name );
				
					else				
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), null, name );
				}
				else
				{
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), null, "" );
				}

				#endregion
			}
		}

		/// <summary>
		///  . 
		///    VALUE   , 
		///       NAME.
		///      .
		/// </summary>
		/// <param name="item">- (CMapperXmlItem)</param>
		/// <returns>  </returns>
		public static string GetValueName ( CMapperXmlItem item )
		{
			if ( item.Value != null )
				return item.Value;
			else
				return item.Name;
		}

		/// <summary>
		///  . 
		///    NAME   , 
		///       VALUE.
		///      .
		/// </summary>
		/// <param name="item">- (CMapperXmlItem)</param>
		/// <returns>  </returns>
		public static string GetNameValue ( CMapperXmlItem item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return item.Value;
		}
	}	
	#endregion

	#region CMapperXmlItemSet
	/// <summary>
	///  XML-,     (  CMapperXmlItem).
	///  XML-   property_set
	///    Name, IdName,        / XML-
	///          CMapperXmlItemSetCollection
	/// </summary>
	[Serializable]
	public class CMapperXmlItemSet : IList
	{
		#region Fields & Properties

		private ArrayList list;
		
		private string m_Name = "";
		/// <summary>
		///     
		///    name  
		/// </summary>
		public string Name
		{
			get { return m_Name; }
			set { m_Name = value; }
		}
		
		private string m_IdName = "";
		/// <summary>
		/// Id    
		///    id  
		/// </summary>
		public string IdName
		{
			get { return m_IdName; }
			set { m_IdName = value; }
		}

		private CMapperXmlItemSetCollection m_ParentCollection = null;
		/// <summary>
		/// -   (ItemSetCollection).
		/// </summary>
		public CMapperXmlItemSetCollection ParentCollection
		{
			get { return m_ParentCollection; }
			set { m_ParentCollection = value; }
		}
		
		private CMapperXmlItemSetCollection m_SubSets = null;
		/// <summary>
		/// 
		/// </summary>
		public CMapperXmlItemSetCollection SubSets
		{
			get { return m_SubSets; }
			set { m_SubSets = value; }
		}

		#endregion

		#region Constructor

		public CMapperXmlItemSet ( ):base ( )
		{
			list = new ArrayList ( );

			m_SubSets = new CMapperXmlItemSetCollection ( );
			m_SubSets.ParentSet = this;
		}
		#endregion

		#region Xml
		/// <summary>
		///      XML 
		///        XML-. 
		///       FromXml  ,
		///           
		/// </summary>
		/// <param name="description">   name   DATA</param>
		/// <param name="iset">  ,    XML-</param>
		/// <returns>  XML-</returns>
		public static string ToXml ( string description, CMapperXmlItemSet iset )
		{
			return CMapperXmlTransformer.GetXmlString ( description, iset );
		}

		/// <summary>
		///     XML   .
		///   ,         .
		///      ToXml  ,    
		/// </summary>
		/// <param name="xml"></param>
		/// <returns></returns>
		public static CMapperXmlItemSet FromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );
			if ( coll.Count > 0 )
				return coll [ 0 ];
			
			return null;
		}

		/// <summary>
		///  ,     
		///        XML-. 
		///       FromXml  ,
		///           
		/// </summary>
		/// <param name="description"></param>
		/// <returns></returns>
		public string WriteToXml ( string description )
		{
			return CMapperXmlTransformer.GetXmlString ( description, this );
		}

		/// <summary>
		///  ,       
		///   ,         .
		///      ToXml  ,    
		/// </summary>
		/// <param name="xml"></param>
		public void ReadFromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );
			if ( coll.Count > 0 )
			{
				CMapperXmlItemSet iset = coll [ 0 ];
			
				this.m_IdName = iset.IdName;
				this.m_Name = iset.Name;				
				this.m_SubSets = iset.SubSets;
				this.list = iset.list;
				//this.m_ParentCollection = iset.ParentCollection;
			}
		}
		#endregion
	
		#region Self Collection Operations
		public int Add ( CMapperXmlItem value )
		{
			if (value == null)
				throw new ArgumentNullException();
			return list.Add ( value );
		}

		public void Insert(int index, CMapperXmlItem value)
		{			
			if (value == null)
				throw new ArgumentNullException();
			list.Insert ( index, value );
		}

		public void Remove ( CMapperXmlItem value )
		{
			if (value == null)
				throw new ArgumentNullException();
			list.Remove ( value );
		}

		public void RemoveAt ( int index )
		{
			list.RemoveAt ( index );
		}
		public bool Contains(CMapperXmlItem value)
		{
			if (value == null)
				return false;
			return list.Contains(value);
		}
		public void Clear()
		{
			list.Clear();
		}
		public int IndexOf(CMapperXmlItem value)
		{			
			if (value == null)
				return -1;
			return list.IndexOf ( value );			
		}
		public CMapperXmlItem this[int index]
		{
			get
			{
				return (CMapperXmlItem) list[index];				
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				list[index] = value;
			}
		}

		#endregion

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return list.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return list.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			list.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return list.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return list.GetEnumerator();
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{				
				return false;
			}
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is CMapperXmlItem ))
					throw new ArgumentException();
				this[index] = (CMapperXmlItem)value;
			}
		}

		void IList.Insert(int index, object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			this.Insert ( index, (CMapperXmlItem)value );
		}

		void IList.Remove(object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			this.Remove((CMapperXmlItem)value);
		}

		bool IList.Contains(object value)
		{			
			if (value == null)
				return false;
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.Contains((CMapperXmlItem)value);
		}
		
		int IList.IndexOf(object value)
		{	
			if (value == null)
				return -1;

			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.IndexOf((CMapperXmlItem)value);
		}

		int IList.Add(object value)
		{
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.Add((CMapperXmlItem)value);			
		}

		public bool IsFixedSize
		{
			get
			{				
				return false;
			}
		}

		#endregion
	}
	#endregion

	#region CMapperXmlItemSetCollection
	/// <summary>
	///  XML-,     CMapperXmlItemSet
	///  XML-   property_collection.
	///    NAME,        / XML-
	///       
	/// </summary>
	[Serializable]
	public class CMapperXmlItemSetCollection : IList
	{
		#region Fields & Properties

		private ArrayList list;
		
		private string m_Name = "";
		/// <summary>
		///     
		///    name  
		/// </summary>
		public string Name
		{
			get { return m_Name; }
			set { m_Name = value; }
		}

		private CMapperXmlItemSet m_ParentSet = null;
		/// <summary>
		/// -   (ItemSet).
		/// </summary>
		public CMapperXmlItemSet ParentSet
		{
			get { return m_ParentSet; }
			set { m_ParentSet = value; }
		}

		#endregion

		#region Constructor

		public CMapperXmlItemSetCollection ( ):base ( )
		{
			list = new ArrayList ( );
		}

		public CMapperXmlItemSetCollection ( string name ):this ( )
		{
			m_Name = name;
		}

		#endregion

		#region Xml
		/// <summary>
		///  . 
		///   XML-     XML    .
		///  ,   .
		///     ( )      DATA
		/// </summary>
		/// <param name="description">,    DATA    NAME</param>
		/// <param name="coll">  ,   </param>
		/// <returns>  XML-</returns>
		public static string ToXml ( string description, CMapperXmlItemSetCollection coll )
		{
			return CMapperXmlTransformer.GetXmlString ( description, coll );
		}

		/// <summary>
		///  
		///   XML-     XML    .
		///     ( )      DATA
		/// </summary>
		/// <param name="description">,    DATA    NAME</param>
		/// <param name="coll">  ,   </param>
		/// <param name="path">   </param>
		public static void ToXmlFile ( string description, CMapperXmlItemSetCollection coll, string path )
		{
			CMapperXmlTransformer.CreateXmlFile ( description, coll, path );
		}

		/// <summary>
		///  . 
		///     XML     ,   
		///     ,   
		/// </summary>
		/// <param name="xml">  XML </param>
		/// <returns> </returns>
		public static CMapperXmlItemSetCollection FromXml ( string xml )
		{
			return CMapperXmlTransformer.ParseXmlString ( xml );			
		}
		
		/// <summary>
		///  . 
		///     XML     ,   
		/// </summary>
		/// <param name="xml">  XML </param>
		/// <returns> </returns>
		public static CMapperXmlItemSetCollection FromXmlFile ( string path )
		{
			return CMapperXmlTransformer.ParseXmlFile ( path );			
		}

		/// <summary>
		///  . 
		///   XML-     XML    .
		///     ( )     XML-  DATA
		/// </summary>
		/// <param name="description">,    DATA    NAME</param>
		/// <returns>  XML-</returns>
		public string WriteToXml ( string description )
		{
			return CMapperXmlTransformer.GetXmlString ( description, this );
		}

		/// <summary>
		///  .
		///    -      XML.
		/// </summary>
		/// <param name="xml">  XML</param>
		public void ReadFromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );

			this.m_Name = coll.Name;
			this.list = coll.list;
			//this.m_ParentSet = coll.ParentSet;
		}
		#endregion

		#region Self
		
		public int Add ( CMapperXmlItemSet value )
		{
			if (value == null)
				throw new ArgumentNullException();
			return list.Add ( value );
		}

		public void Insert(int index, CMapperXmlItemSet value)
		{			
			if (value == null)
				throw new ArgumentNullException();
			list.Insert ( index, value );
		}

		public void Remove ( CMapperXmlItemSet value )
		{
			if (value == null)
				throw new ArgumentNullException();
			list.Remove ( value );
		}

		public void RemoveAt ( int index )
		{
			list.RemoveAt ( index );
		}
		public bool Contains(CMapperXmlItemSet value)
		{
			if (value == null)
				return false;
			return list.Contains(value);
		}
		public void Clear()
		{
			list.Clear();
		}
		public int IndexOf(CMapperXmlItemSet value)
		{			
			if (value == null)
				return -1;
			return list.IndexOf ( value );			
		}
		public CMapperXmlItemSet this[int index]
		{
			get
			{
				return (CMapperXmlItemSet) list[index];				
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				list[index] = value;
			}
		}

		#endregion			

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return list.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return list.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			list.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return list.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return list.GetEnumerator();
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{				
				return false;
			}
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is CMapperXmlItemSet ))
					throw new ArgumentException();
				this[index] = (CMapperXmlItemSet)value;
			}
		}

		void IList.Insert(int index, object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			this.Insert ( index, (CMapperXmlItemSet)value );
		}

		void IList.Remove(object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			this.Remove((CMapperXmlItemSet)value);
		}

		bool IList.Contains(object value)
		{			
			if (value == null)
				return false;
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.Contains((CMapperXmlItemSet)value);
		}
		
		int IList.IndexOf(object value)
		{	
			if (value == null)
				return -1;

			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.IndexOf((CMapperXmlItemSet)value);
		}

		int IList.Add(object value)
		{
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.Add((CMapperXmlItemSet)value);			
		}

		public bool IsFixedSize
		{
			get
			{				
				return false;
			}
		}

		#endregion
	}
	#endregion

	#region CSQLAnswerParser Ruzil
	public class CSQLAnswerParser
	{
		#region SQLDataValue -  :)
		public class SQLDataValue
		{
			public object m_Value=null;
			public object Value
			{
				set{m_Value = value;}
				get{return m_Value;}
			}

			public override string ToString()
			{
				if (m_Value == null)
					return "NULL";
				else
					return Value.ToString();
			}


			public SQLDataValue()
			{
			}
		}
		#endregion

		#region SQLDataColumn -  SQLDataValue
		public class SQLDataColumn : IList
		{
			private	ArrayList	List;
			public string Name = "";
			public string TypeName = "";

			#region 
			public SQLDataColumn ( ):base ( )
			{
				List = new ArrayList ();
			}

			public int Add ( SQLDataValue value )
			{
				if (value == null)
					throw new ArgumentNullException();
				return List.Add ( value );
			}

			public void Insert(int index, SQLDataValue value)
			{			
				if (value == null)
					throw new ArgumentNullException();
				List.Insert ( index, value );
			}

			public void Remove ( SQLDataValue value )
			{
				if (value == null)
					throw new ArgumentNullException();
				List.Remove ( value );
			}

			public void RemoveAt ( int index )
			{
				List.RemoveAt ( index );
			}

			public bool Contains(SQLDataValue value)
			{
				if (value == null)
					return false;
				return List.Contains(value);
			}

			public void Clear()
			{
				List.Clear();
			}

			public int IndexOf(SQLDataValue value)
			{			
				if ( value == null )
					return -1;
				return List.IndexOf ( value );			
			}

			public SQLDataValue this [ int index ]
			{
				get
				{
					if ( List.Count < index )
						return (SQLDataValue) List [index];				
					else
						return null;
				}
				set
				{
					if ( value == null )
						throw new ArgumentNullException();
					List [ index ] = value;
				}
			}

			#endregion

			#region ICollection Members

			public bool IsSynchronized
			{
				get
				{
					return List.IsSynchronized;
				}
			}

			public int Count
			{
				get
				{
					return List.Count;
				}
			}

			public void CopyTo ( Array array, int index )
			{
				List.CopyTo(array, index);
			}

			public object SyncRoot
			{
				get
				{
					return List.SyncRoot;
				}
			}

			#endregion

			#region IEnumerable Members

			public IEnumerator GetEnumerator()
			{
				return List.GetEnumerator();
			}

			#endregion

			#region IList Members

			public bool IsReadOnly
			{
				get
				{				
					return false;
				}
			}

			object IList.this[int index]
			{
				get
				{
					return this[index];
				}
				set
				{
					if (value == null)
						throw new ArgumentNullException();
					if (!( value is SQLDataValue ))
						throw new ArgumentException();
					this[index] = (SQLDataValue)value;
				}
			}

			void IList.Insert(int index, object value)
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is SQLDataValue ))
					throw new ArgumentException();
				this.Insert ( index, (SQLDataValue)value );
			}

			void IList.Remove(object value)
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is SQLDataValue ))
					throw new ArgumentException();
				this.Remove((SQLDataValue)value);
			}

			bool IList.Contains(object value)
			{			
				if (value == null)
					return false;
				if (!( value is SQLDataValue ))
					throw new ArgumentException();
				return this.Contains((SQLDataValue)value);
			}
		
			int IList.IndexOf(object value)
			{	
				if (value == null)
					return -1;

				if (!( value is SQLDataValue ))
					throw new ArgumentException();
				return this.IndexOf((SQLDataValue)value);
			}

			int IList.Add(object value)
			{
				if (!( value is SQLDataValue ))
					throw new ArgumentException();
				return this.Add((SQLDataValue)value);			
			}

			public bool IsFixedSize
			{
				get
				{				
					return false;
				}
			}

			#endregion
		}
		#endregion

		#region SQLDataAnswer -  SQLDataColumn
		public class SQLDataAnswer : IList
		{
			private	ArrayList	List;

			#region 
			public SQLDataAnswer ( ):base ( )
			{
				List = new ArrayList ();
			}

			public int Add ( SQLDataColumn value )
			{
				if (value == null)
					throw new ArgumentNullException();
				return List.Add ( value );
			}

			public void Insert(int index, SQLDataColumn value)
			{			
				if (value == null)
					throw new ArgumentNullException();
				List.Insert ( index, value );
			}

			public void Remove ( SQLDataColumn value )
			{
				if (value == null)
					throw new ArgumentNullException();
				List.Remove ( value );
			}

			public void RemoveAt ( int index )
			{
				List.RemoveAt ( index );
			}

			public bool Contains(SQLDataColumn value)
			{
				if (value == null)
					return false;
				return List.Contains(value);
			}

			public void Clear()
			{
				List.Clear();
			}

			public int IndexOf(SQLDataColumn value)
			{			
				if ( value == null )
					return -1;
				return List.IndexOf ( value );			
			}

			public SQLDataColumn this [ int index ]
			{
				get
				{
					return (SQLDataColumn) List [index];				
				}
				set
				{
					if ( value == null )
						throw new ArgumentNullException();
					List [ index ] = value;
				}
			}

			#endregion

			#region ICollection Members

			public bool IsSynchronized
			{
				get
				{
					return List.IsSynchronized;
				}
			}

			public int Count
			{
				get
				{
					return List.Count;
				}
			}

			public void CopyTo ( Array array, int index )
			{
				List.CopyTo(array, index);
			}

			public object SyncRoot
			{
				get
				{
					return List.SyncRoot;
				}
			}

			#endregion

			#region IEnumerable Members

			public IEnumerator GetEnumerator()
			{
				return List.GetEnumerator();
			}

			#endregion

			#region IList Members

			public bool IsReadOnly
			{
				get
				{				
					return false;
				}
			}

			object IList.this[int index]
			{
				get
				{
					return this[index];
				}
				set
				{
					if (value == null)
						throw new ArgumentNullException();
					if (!( value is SQLDataColumn ))
						throw new ArgumentException();
					this[index] = (SQLDataColumn)value;
				}
			}

			void IList.Insert(int index, object value)
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is SQLDataColumn ))
					throw new ArgumentException();
				this.Insert ( index, (SQLDataColumn)value );
			}

			void IList.Remove(object value)
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is SQLDataColumn ))
					throw new ArgumentException();
				this.Remove((SQLDataColumn)value);
			}

			bool IList.Contains(object value)
			{			
				if (value == null)
					return false;
				if (!( value is SQLDataColumn ))
					throw new ArgumentException();
				return this.Contains((SQLDataColumn)value);
			}
		
			int IList.IndexOf(object value)
			{	
				if (value == null)
					return -1;

				if (!( value is SQLDataColumn ))
					throw new ArgumentException();
				return this.IndexOf((SQLDataColumn)value);
			}

			int IList.Add(object value)
			{
				if (!( value is SQLDataColumn ))
					throw new ArgumentException();
				return this.Add((SQLDataColumn)value);			
			}

			public bool IsFixedSize
			{
				get
				{				
					return false;
				}
			}

			#endregion
		}
		#endregion

		public CSQLAnswerParser()
		{
		}
		public static SQLDataAnswer Parse(string xml)
		{
			#region  xml
			Hashtable hash = new Hashtable ();
			CMapperXmlItemSetCollection coll = CMapperXmlItemSetCollection.FromXml ( xml );
			if ( coll == null )
			{
				return null;
			}
			else	
			{
				coll = coll [ 0 ].SubSets;
				CMapperXmlItemSetCollection coll2;
				CMapperXmlItemSetCollection coll3;

				if ( coll.Count < 2 )
					return null;
				else
				{
					CMapperXmlItemSet headset = null;
					CMapperXmlItemSet dataset = null;

					for ( int i = 0; i < coll.Count; i++ )
					{
						CMapperXmlItemSet tset = coll [ i ];
						switch ( tset.Name.ToLower() )
						{
							case "tableheader":
								headset = tset;
								break;

							case "tabledata":
								dataset = tset;
								break;
						}
					}

					if (( headset != null ) && ( dataset != null ))
					{
						#region  
						coll = headset.SubSets;
						//  
						for ( int i = 0; i < coll.Count; i++ )
						{
							CMapperXmlItemSet tset = coll [ i ];
							if (tset.Count > 0 )
							{
								string tmps = CMapperXmlItem.GetValueName ( tset[0] );
								string Id = tset.IdName;
								string Value = "";
								string TypeName = "";
								for ( int i4 = 0; i4 < tset.Count; i4++ )
								{
									switch ( tset[i4].Key.ToLower() )
									{
										case "name":
											Value = CMapperXmlItem.GetValueName (tset[i4] );
											break;
										case "typename":
											TypeName = CMapperXmlItem.GetValueName (tset[i4] );
											break;
									}
								}

								SQLDataColumn column = new SQLDataColumn ();
								column.Name = Value;
								column.TypeName = TypeName;
								hash[Id] = column;
								//hash [ Id ] = Value;

							}
						}
						#endregion

						#region  
						coll2 = dataset.SubSets;
						//  
						if ( coll2.Count < 2 )
							return null;

						CMapperXmlItemSet xset = coll2 [ coll2.Count-1 ];
						coll3 = xset.SubSets;
						//  
						for ( int i2 = 0; i2 < coll2.Count; i2++ )
						{						
							CMapperXmlItemSet tset = coll2 [ i2 ];
							coll3 = tset.SubSets;
							//  
							for ( int i3 = 0; i3 < coll3.Count; i3++ )
							{
								tset = coll3 [ i3 ];
								if (tset.Count > 0 )
								{
									string tmps = CMapperXmlItem.GetValueName ( tset[0] );


									SQLDataColumn column = hash [tset.IdName] as SQLDataColumn;
									if (column != null )
									{
										SQLDataValue sqlValue = new SQLDataValue ();
										switch ( column.TypeName.ToLower() )
										{
											case "int32":
												try
												{
													sqlValue.Value = int.Parse(tmps);
												}
												catch
												{
													sqlValue.Value = 0;
												}
												break;
											case "guid":
												try
												{
													sqlValue.Value = new Guid (tmps);
												}
												catch
												{
													sqlValue.Value = Guid.Empty;
												}
												break;
											case "string":
											default:
												sqlValue.Value = tmps;
												break;
										}
										column.Add ( sqlValue );
									}
								}
							}
						}
						#endregion
					}
				}

			}
			#endregion


			if ( hash != null )
			{
				SQLDataAnswer sda = new SQLDataAnswer ();
				foreach (DictionaryEntry de in hash)
				{
					sda.Add((SQLDataColumn) de.Value);
				}
				return sda;
			}
			else
				return null;
			//source.Add ( field );

		}
	}
	#endregion
}
